#include <sta/tacos/system/events.hpp>
#include <sta/rtos/event.hpp>
#include <sta/rtos/timer.hpp>

#include <sta/debug/assert.hpp>

#include <cmsis_os2.h>
#include <FreeRTOS.h>


namespace
{
	// Event handle
	sta::RtosEvent * systemEvents = nullptr;

	// Tick timer handle
	sta::RtosTimer * tickTimer = nullptr;

	uint8_t tickCounter = 0;
}


namespace sta
{
	namespace tacos
	{
		namespace events
		{
			void init()
			{
				if (systemEvents == nullptr)
				{
					systemEvents = new sta::RtosEvent();
				}

				if (tickTimer == nullptr)
				{
					tickTimer = new sta::RtosTimer([](void * args) {
						uint32_t flags = 0x00;

						flags |= STA_TACOS_TOCK_50_Hz;

						if (tickCounter % 20 == 0) {
							flags |= STA_TACOS_TICK_50_Hz;
						} else {
							flags |= STA_TACOS_TOCK_50_Hz;
						}

						if (tickCounter % 50 == 0) {
							flags |= STA_TACOS_TICK_20_Hz;
						} else if (tickCounter % 25 == 0) {
							flags |= STA_TACOS_TOCK_20_Hz;
						}

						if (tickCounter % 100) {
							flags |= STA_TACOS_TICK_10_Hz;
						} else if (tickCounter % 50 == 0) {
							flags |= STA_TACOS_TOCK_10_Hz;
						}

						if (tickCounter % 1000) {
							flags |= STA_TACOS_TICK_1_Hz;
						} else if (tickCounter % 500 == 0) {
							flags |= STA_TACOS_TOCK_1_Hz;
						}

						tickCounter = (tickCounter + 10) % 1000;

						signal(flags);
						systemEvents->clear(STA_TACOS_EVENT_TIMER_MASK);
					}, nullptr, true);

					tickTimer->start(10);
				}
			}

			void signal(uint32_t flags)
			{
				STA_ASSERT_MSG(systemEvents != nullptr, "System events not initialized");
				systemEvents->set(flags);
			}

			void wait(uint32_t flags, uint32_t options, uint32_t timeout)
			{
				STA_ASSERT_MSG(systemEvents != nullptr, "System events not initialized");
				systemEvents->peek(flags, timeout);
			}
	
			void signalStartup()
			{
				signal(STA_TACOS_SYSTEM_EVENTS_STARTUP);
			}

			void waitForStartup()
			{
				wait(STA_TACOS_SYSTEM_EVENTS_STARTUP, osFlagsWaitAll, osWaitForever);
			}
		}  // namespace events
	} // namespace tacos
} // namespace sta
